from .test_io import *
from .test_basics import *
from .test_operations import *
from .test_normalize import *
from .test_binning import *
from .test_simulations import *
from .test_visualizations import *
from .test_multi import *
from .test_resampling import *
from .test_units import *
from .test_align_wavelengths import *
from .test_shift import *
from .test_conversions import *
from .test_summaries import *
from .test_withmodel import *
from .test_ok import *
from .test_history import *
from .test_spectra import *
from .test_remove_trends import *
from .test_time import *
from .test_fold import *
